program Manhattan (input, output);

const
   max = 100;

type
   { or\"ientatie co\"ordinaten:
    ---> 2e dim. <==> lanes
    |
    |
    1e dim. <==> avenues }
   rooster = array[1..max, 1..max] of boolean;
      { true <--> kruispunt toegankelijk
       false <--> kruispunt afgesloten }
   coord = record
      avenue, lane : integer;
   end;
   statistiek = array[1..max, 1..max] of record
      som, aantal : integer;
   end;

var
   stad, nsteden,          { run-variabelen }
   navenues,               { #avenues }
   nlanes : integer;       { #lanes }
   kaart : rooster;
   histo : statistiek;


procedure leesKaart (var navenues, nlanes : integer; var kaart : rooster);
{ lees kaart in en sla die op }

var
   ch   : char;
   a, l : integer;     { avenues & lanes }

begin
   readln(navenues, nlanes);
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 read(ch);
	 kaart[a, l] := (ch='+');
      end;
      readln;
   end;
end; { leesKaart }


function viaAvenue (pos : coord; var naar : coord) : boolean;
{ checkt of er een andere lane via een avenue te bereiken valt
 en zoja, welke }

begin
   if odd(pos.avenue) then begin
      if pos.lane > 1 then begin
	 naar.avenue := pos.avenue;
	 naar.lane := pos.lane-1;
	 viaAvenue := kaart[naar.avenue, naar.lane];
      end else begin
	 viaAvenue := false;
      end;
   end else begin
      if pos.lane < nlanes then begin
	 naar.avenue := pos.avenue;
	 naar.lane := pos.lane+1;
	 viaAvenue := kaart[naar.avenue, naar.lane];
      end else begin
	 viaAvenue := false;
      end;
   end;
end; { viaAvenue }


function viaLane (pos : coord; var naar : coord) : boolean;
{ checkt of er een andere avenue via een lane te bereiken valt
 en zoja, welke }

begin
   if odd(pos.lane) then begin
      if pos.avenue < navenues then begin
	 naar.avenue := pos.avenue+1;
	 naar.lane := pos.lane;
	 viaLane := kaart[naar.avenue, naar.lane];
      end else begin
	 viaLane := false;
      end;
   end else begin
      if pos.avenue > 1 then begin
	 naar.avenue := pos.avenue-1;
	 naar.lane := pos.lane;
	 viaLane := kaart[naar.avenue, naar.lane];
      end else begin
	 viaLane := false;
      end;
   end;
end; { viaLane }


procedure wandel (a, l : integer; var histo : statistiek);

const
   kruismax = 10000;

var
   queue   : array[1..kruismax] of coord;
   minstap : array[1..max, 1..max] of integer;
   stap,                  { lengte van het pad }
   beginQ,                { index begin queue }
   eindQ,                 { index einde queue }
   lengteQ,               { lengte nieuwe queue }
   i, j, p : integer;     { hulp-variabelen }
   naar : coord;          { hulp-variabele }

begin
   { initialiseer minimum-stappen-matrix }
   for i := 1 to navenues do begin
      for j := 1 to nlanes do begin
	 minstap[i, j] := 0;
      end;
   end;
   minstap[a, l] := -1;  { niet mee laten tellen }
   { initialiseer queue door huidige kruispunt erop te plaatsen }
   stap := 0;
   beginQ := 1;
   eindQ := 1;
   lengteQ := 1;
   queue[1].avenue := a;
   queue[1].lane := l;
   while lengteQ > 0 do begin
      lengteQ := 0;
      stap := stap+1;
      for p := beginQ to eindQ do begin
	 { haal kruispunt van de queue en ga naar aangrenzende kruispunten }
	 if viaAvenue(queue[p], naar) then begin
	    if minstap[naar.avenue, naar.lane] = 0 then begin
	       minstap[naar.avenue, naar.lane] := stap;
	       lengteQ := lengteQ+1;
	       queue[eindQ+lengteQ] := naar;
	    end;
	 end;
	 if viaLane(queue[p], naar) then begin
	    if minstap[naar.avenue, naar.lane] = 0 then begin
	       minstap[naar.avenue, naar.lane] := stap;
	       lengteQ := lengteQ+1;
	       queue[eindQ+lengteQ] := naar;
	    end;
	 end;
      end;
      { verplaats queue-indices }
      beginQ := eindQ + 1;
      eindQ := beginQ + lengteQ - 1;
   end;
   { werk statistieken bij }
   for i := 1 to navenues do begin
      for j := 1 to nlanes do begin
	 if minstap[i, j] > 0 then begin
	    with histo[i, j] do begin
	       som := som + minstap[i, j];
	       aantal := aantal + 1;
	    end;
	 end;
      end;
   end;
end; { wandel }


procedure wandelAlle (var histo : statistiek);
{ ga alle bereikbare kruispunten af: alle-paren kortste pad }

var
   a, l : integer;  { loop-variabelen voor avenues & lanes }

begin
   { initialiseer statistieken }
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 with histo[a, l] do begin
	    som := 0;
	    aantal := 0;
	 end;
      end;
   end;
   { voor alle kruispunten: ... }
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 if kaart[a, l] then begin
	    wandel(a, l, histo);
	 end;
      end;
   end;
   { DEBUG :: }
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 if kaart[a, l] then begin
	    write(histo[a, l].aantal:3, '/', histo[a, l].som:4);
	 end else begin
	    write(' ':8);
	 end;
	 write('  ');
      end;
      writeln;
   end;
   writeln;
end; { wandelAlle }
  

procedure schrijfHistogram;

var
   a, l,     { loop-variabelen voor avenues & lanes }
   maxsom, maxaantal : integer; { maximale bereikbaarheid }

begin
   { bepaal maximale bereikbaarheid }
   maxsom := 1;
   maxaantal := 0;
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 with histo[a, l] do begin
	    if (aantal > 0) and (aantal*maxsom > som*maxaantal) then begin
	       maxsom := som;
	       maxaantal := aantal;
	    end;
	 end;
      end;
   end;
   { druk af }
   writeln('MAX: ', maxaantal:1, '/', maxsom:1);
   writeln;
   for a := 1 to navenues do begin
      for l := 1 to nlanes do begin
	 if not kaart[a, l] then begin
	    write('x');
	 end else begin
	    with histo[a, l] do begin
	       if aantal = 0 then begin
		  write('Z');
	       end else begin
		  if 3*maxsom*aantal < maxaantal*som then begin
		     write('R');
		  end else begin
		     if 3*maxsom*aantal >= 2*maxaantal*som then begin
			write('G');
		     end else begin
			write('O');
		     end;
		  end;
	       end;
	    end;
	 end;
      end;
      writeln;
   end;
end; { schrijfHistogram }


begin
   readln(nsteden);
   for stad := 1 to nsteden do begin
      leesKaart(navenues, nlanes, kaart);
      wandelAlle(histo);
      schrijfHistogram;
      if stad<nsteden then begin
	 writeln;
      end;
   end;
end.
