#include <stdlib.h>
#include <stdio.h>

#define MAXKNOT 300

typedef struct edge {
  int from,label,to;
  int sims[MAXKNOT];
  struct edge *nextchild,*nextparent;
} edge;

typedef struct graph {
  int knots;
  edge *child[MAXKNOT],*parent[MAXKNOT];
} graph;

graph g1,g2;
int doessim[MAXKNOT][MAXKNOT];
int propagated[MAXKNOT][MAXKNOT];

void ReadGraph(graph *g) {
  int edges;
  edge *ne; /* new edge */
  int i,j;
  scanf("%d %d",&(g->knots),&edges);
  for (i=0;i<g->knots;i++) {
    g->child[i]=NULL;
    g->parent[i]=NULL;
  }
  for (i=0;i<edges;i++) {
    ne=malloc(sizeof(edge));
    scanf("%d %d %d",&ne->from,&ne->label,&ne->to);
    for (j=0;j<MAXKNOT;j++) ne->sims[j]=0;
    ne->nextchild=g->child[ne->from];
    g->child[ne->from]=ne;
    ne->nextparent=g->parent[ne->to];
    g->parent[ne->to]=ne;
  }
}

void ReadInit() {
  int i,j;
  ReadGraph(&g1);
  ReadGraph(&g2);
  for (i=0;i<g1.knots;i++) {
    for (j=0;j<g2.knots;j++) {
      doessim[i][j]=1;
      propagated[i][j]=0;
    }
  }
}

void Propagate(int i,int j) {
  edge *p1,*p2; /* ancestors of g1[i] resp. g2[j] */
#ifdef DEBUG
  fprintf(stderr,"Prop(%d,%d); ",i,j);
#endif
  doessim[i][j]=0;
  propagated[i][j]=1;
  p1=g1.parent[i];
  while (p1!=NULL) {
    p2=g2.parent[j];
    while (p2!=NULL) {
      if (p1->label==p2->label) {
	p1->sims[p2->from]--;
	if (p1->sims[p2->from]==0) {
	  if (!propagated[p1->from][p2->from]) Propagate(p1->from,p2->from);
	}
      }
      p2=p2->nextparent;
    }
    p1=p1->nextparent;
  }
}

void Simulate() {
  int i,j;
  edge *e1,*e2;
  for (i=0;i<g1.knots;i++) {
    for (j=0;j<g2.knots;j++) {
      e1=g1.child[i];
      while (e1!=NULL) {
	e2=g2.child[j];
	while (e2!=NULL) {
	  if (e2->label==e1->label) e1->sims[j]++;
	  e2=e2->nextchild;
	}
	e1=e1->nextchild;
      }
    }
  }
  for (i=0;i<g1.knots;i++) {
    e1=g1.child[i];
    while (e1!=NULL) {
      for (j=0;j<g2.knots;j++) {
#ifdef DEBUG
	fprintf(stderr,"[%d,%d:%d-%d-%d:%d]\n",
		i,j,e1->from,e1->label,e1->to,e1->sims[j]);
#endif
	if (e1->sims[j]==0) {
	  if (!propagated[i][j]) Propagate(i,j);
	}
      }
      e1=e1->nextchild;
    }
  }
}

void QuitEdge(edge *e) {
  if (e!=NULL) {
    QuitEdge(e->nextchild); /* EITHER all children OR all parents */
    free(e);
  }
}

void QuitGraph(graph *g) {
  int i;
  for (i=0;i<g->knots;i++) {
    QuitEdge(g->child[i]); /* EITHER child OR parent, match QuitEdge() */
  }
}

void PrintQuit() {
  int i,j,first=1;
  for (i=0;i<g1.knots;i++) {
    for (j=0;j<g2.knots;j++) {
      if (doessim[i][j]) {
	if (first) first=0; else printf("  ");
	printf("%d %d",i,j);
      }
    }
  }
  printf("\n");
  QuitGraph(&g1);
  QuitGraph(&g2);
}

int main() {
  int runs,r;
  scanf("%d",&runs);
  for (r=0;r<runs;r++) {
    ReadInit();
    Simulate();
    PrintQuit();
  }
  return(0);
}
