/*
   RSA -- berekent b^c mod a waarbij a, b en c grote base-26 getallen zijn

   (c) 28/5/97 Gerton Lunter, voor het NKP '97
   (a) 27/9/97 Meinte Boersma
*/


#include <stdio.h>

#define maxdig 82

typedef char getal[maxdig];
typedef getal rij[2*maxdig];

rij macht;     /* macht[i] == 26^i modulo mod */
getal nul;     /* == 0 in base-26 */


char deel2 (getal x)
     /* x := x/2 and deel2 = x modulo 2 */
{
  int i = maxdig;
  char c = 0;

  while (i) {
    i--;
    x[i] = (c = (x[i] + 26*(c&1))) >> 1;
  }
  return (c&1);
}


char vergelijk (getal x, getal y)
     /* levert 1 als x>y, -1 als x<y, anders 0 */
{
  int i = maxdig, result = 0;

  while (i) {
    i--;
    if (x[i] > y[i]) {
      result = 1;
      i = 0;
    } else {
      if (x[i] < y[i]) {
	result = -1;
	i = 0;
      }
    }
  }
  return(result);
}


void trekaf (getal x, getal y, getal z)
     /* z := x - y */
{
  int i = 0;
  int carry = 0;
  
  while (i<maxdig) {
    z[i] = x[i] - y[i] + carry;
    if (z[i]<0) {
      z[i] += 26;
      carry = -1;
    } else {
      carry = 0;
    }
    i++;
  }
}


void initmacht (getal mod, rij macht)
     /* initialiseert macht-array */
{
  int i, j;

  for (i=0; i<maxdig; i++) {
    macht[0][i] = 0;
  }
  macht[0][0]=1;

  for (j=1; j<2*maxdig; j++) {
    for (i=1; i<maxdig; i++) {
      macht[j][i] = macht[j-1][i-1];
    }
    macht[j][0] = 0;
    while (vergelijk(macht[j], mod) >= 0) {
      trekaf(macht[j], mod, macht[j]);
    }
  }
}


void keer (getal x, getal y, getal mod, getal z)
     /* z := x*y modulo mod */
{
  unsigned int tussen[2*maxdig];
  unsigned int tmp;
  char verandering;
  int i,j;

  /* doen */

  tussen[0] = 0;
  for (i=0; i<2*maxdig-1; i++) {
    for (j=0; j<maxdig; j++) {
      if ((i-j>=0) && (i-j<maxdig))
	tussen[i] += x[i-j]*y[j];
    }
    tussen[i+1] = tussen[i]/26;
    tussen[i] %= 26;
  }

  /* nu modulus nemen & carry rippelen net zolang tot er niets meer verandert */

  do {
    verandering = 0;
    for (i=0; i<2*maxdig; i++) {
      tmp = tussen[i];
      if (tmp && ((i>=maxdig) || (!macht[i][i]))) {
	tussen[i] = 0;
	verandering = 1;
	for (j=0; j<maxdig; j++) {
	  tussen[j] += tmp*(unsigned int)macht[i][j];
	}
      }
    }
    for (i=0; i<2*maxdig-1; i++) {
      if (tussen[i]/26) {
	verandering = 1;
      }
      tussen[i+1] += tussen[i]/26;
      tussen[i] %= 26;
    }
  } while (verandering);

  /* resultaat opslaan & meest significante digit "moduleren" */

  for (i=0; i<maxdig; i++) {
    z[i] = tussen[i];
  }
  while (vergelijk(z, mod) >= 0) {
    trekaf(z, mod, z);
  }
}


void neemmacht (getal x, getal y, getal mod, getal z)
     /* z := x^y modulo mod */
{
  int i;
  getal xtot2toti;

  for (i=0; i<maxdig; i++) {
    z[i] = 0;
    xtot2toti[i] = x[i];
  }
  z[0] = 1;

  while (vergelijk(y, nul)) {
    if (deel2(y)) {
      keer(z, xtot2toti, mod, z);
    }
    keer(xtot2toti, xtot2toti, mod, xtot2toti);
  }
}


void leesgetal (getal x)
{
  int i = maxdig, j;
  char c;

  do {
    scanf("%c", &c);
    x[--i] = c - 'a';
  } while (c != '\n');
  j = 0;
  do {
    x[j++] = x[++i];
  } while (i<maxdig-1);
  do {
    x[j++] = 0;
  } while (j<maxdig);
}


void schrijfgetal (getal x)
{
  int i = maxdig;

  do {
    i--;
  } while (i && !x[i]);
  do {
    printf("%c", x[i--] + 'a');
  } while (i+1);
  printf("\n");
}


int main (void)
{
  int nruns, run;
  getal a, b, c, bericht;

  /* zero... */
  {
    int i=0;
    for (; i<maxdig; i++) {
      nul[i]=0;
    }
  }

  /* runs */
  scanf("%d ", &nruns);
  for (run=0; run<nruns; run++) {
    leesgetal(a);
    leesgetal(b);
    leesgetal(c);
    initmacht(a, macht);
    neemmacht(b, c, a, bericht);
    schrijfgetal(bericht);
  }

  /* exit */
  return(0);
}
