/*
   semantics-checker for "Turing-compiler"
   (c) 12/10/'97 Meinte Boersma
*/

#include <stdio.h>
#include <stdarg.h>

/* exit-code for jury-malfunctions */
#define rogue_exit_code 999

/* limits for the simulator */
#define maxstate 10000
#define maxsteps 5000
#define maxtape 100

/* alphabet <--> ordinals */
#define alphabet(ord) ((ord!=7)?(ord+'a'):'~')
#define inv_alphabet(symb) ((symb!='~')?(symb-'a'):7)

/* macros for tape-management */
#define read_tape(x, c) c = tape[x+maxtape];
#define write_tape(x, c) tape[x+maxtape] = c;
#define tape_within_limits ( (-maxtape<=pos) && (pos<=maxtape) )
#define show_intermediate 0 /* show intermediate tape-contents */
#define mark_tape 0 /* show | on 0-position */

/* tmp-variables */
#define decl_tmp int i; transsymb *p


typedef enum { false, true } boolean;
typedef enum {
  halted, nonexisting_state, too_many_steps, out_of_tape
} term_type;

typedef struct {
  int newstate;
  char newsymb;
  char dir;
} transsymb;
/* holds transition-description for one alphabet-character */

int nstates;     /* number of states */
transsymb transfunc[maxstate][8];
char tape[2*(maxtape+1)];

FILE *tape_file;

     
void read_transfunc (const char *fname)
{
  FILE *fp;
  decl_tmp;
  char ch;

  fp = fopen(fname, "r");
  if (fp == NULL) {
    fprintf(stderr, "file %s not readable!\n", fname);
    exit(rogue_exit_code);   /* something really wrong! */
  } else {
    /* read 1st line (table-header) */
    while (ch = getc(fp), ch != '\n') {};
    /* read table */
    nstates = 0;
    while (ch != '*') {
      ungetc(ch, fp);
      fscanf(fp, "%d", &i); /* i as a dummy */
      for (i=0; i<8; i++) {
	p = &(transfunc[nstates][i]);
	fscanf(fp, "%d%c%c", &(p->newstate), &(p->newsymb), &(p->dir));
      }
      nstates++;
      ch = getc(fp);
    }
    fclose(fp);
  }
  nstates--;
}


void show_transfunc (void)
{
  int state;
  decl_tmp;

  fprintf(stderr, "nstates=%d\n", nstates);
  printf(" q      a      b      c      d      e      f      g      ~\n");
  for (state=0; state<nstates-1; state++) {
    printf("%4d ", state);
    for (i=0; i<8; i++) {
      p = &(transfunc[state][i]);
      printf("%4d%c%c ", p->newstate, p->newsymb, p->dir);
    }
    printf("\n");
  }
  printf("*\n");
}


boolean init_tape (void)
{
  int pos;
  char ch;

  ch = getc(tape_file);
  if (feof(tape_file)) {
    return(false);
  } else {
    ungetc(ch, tape_file);
    /* empty entire tape */
    for (pos=-maxtape; pos<=maxtape; pos++) {
      write_tape(pos, '~');
    }
    write_tape(maxtape+1, '\0');
    /* read input from file */
    pos = 0;
    while (fscanf(tape_file, "%c", &ch), ch != '\n') {
      write_tape(pos, ch);
      pos++;
    }
    return(true);
  }
}


void show_tape (void)
{
#if (!markt_tape)
  printf("%s\n", tape);
#else mark_tape
  int pos;

  printf("%s\n", tape);
  for (pos=-maxtape; pos<0; pos++) {
    printf(" ");
  }
  printf("|");
  for (; pos<maxtape; pos++) {
    printf(" ");
  }
  printf("\n");
#endif
}


void show_result (void)
{
  int pos = 0;
  char ch;

  read_tape(pos, ch)
  while (ch != '~') {
    printf("%c", ch);
    pos++;
    read_tape(pos, ch);
  }
  printf("\n");
}


void show_status (int state, int nsteps, int pos)
{
  int i;

  printf("%s\n", tape);
  for (i=-maxtape; i<pos; i++) {
    printf(" ");
  }
  printf("^\n");
  for (i=-maxtape; i<pos; i++) {
    printf(" ");
  }
  printf("| state=%d nsteps=%d\n\n", state, nsteps);
}


term_type simulate_TM (void)
{
  boolean cont = true, halt = false;
  int state = 0, pos = 0, nsteps = 0;
  transsymb *p;
  char symb;

#if show_intermediate
  show_tape();
#endif
  while ( cont && (nsteps<maxsteps) ) {
    nsteps++;
    read_tape(pos, symb);
    p = &(transfunc[state][inv_alphabet(symb)]);
    state = p->newstate;
    write_tape(pos, p->newsymb);
    if (symb != p->newsymb) {
      fprintf(stderr, "%c", p->newsymb);
    }
    switch (p->dir) {
      case 'L' : pos--;	fprintf(stderr, "L"); break;
      case 'R' : pos++; fprintf(stderr, "R"); break;
      case 'h' : halt = true; break;
      case '.' : break;     /* '.' does nothing */
      default : fprintf(stderr, "?\n"); break;
    }
    cont = ( tape_within_limits && (state<nstates) && !halt );
#if show_intermediate
    show_status(state, nsteps, pos);
#endif
  }
  fprintf(stderr, "\n");

  if (halt) {
    return (halted);
  } else {
    if (nsteps>=maxsteps) {
      return (too_many_steps);
    } else {
      if (!tape_within_limits) {
	return (out_of_tape);
      } else {
	return (nonexisting_state);
      }
    }
  }
}


int main (int argc, char *argv[])
{
/*int exit_code;*/

  if (argc != 3) {
    fprintf(stderr, "   --- arg 1: file for transition-function\n");
    fprintf(stderr, "   --- arg 2: file for Turing-tape(s)\n");
    exit(rogue_exit_code);   /* something really wrong! */
  } else {
    /* open tape-file */
    tape_file = fopen(argv[2], "r");
    if (tape_file == NULL) {
      fprintf(stderr, "file %s not readable!\n", argv[2]);
      exit(rogue_exit_code);
    }
    read_transfunc(argv[1]);
    while (init_tape()) {
      switch (simulate_TM()) {
	case halted : show_result(); break;
	case too_many_steps : printf("too many steps\n"); break;
	case out_of_tape : printf("ran out of tape\n"); break;
	case nonexisting_state : printf("nonexisting state\n"); break;
      }
    }
    fclose(tape_file);
    return(0);
  }
}
