program syntaxchecker (input, output);

type
   syntaxtype = (correct, wrong_format, incorrect);
   charset = set of char;
   readstatus = record
      { in: }
      alphabet : charset;        { admissible alphabet }
      spacescount : boolean;     { spaces to be counted or not }
      { out: }
      eofreached,                { EOF reached after reading alphabet-character
				  or spaces if spacescount=true or reading EOLN }
      eolnreached,               { EOLN reached after reading alphabet-characters
				  or spaces if spacescount=true }
      succes : boolean;          { read alphabet-character succesfully }
      nspaces : integer;         { case spacescount of
				  true : nspaces="#spaces read before alphabet-character or EOF/EOLN";
				  false : nspaces undefined }
      chread : char;             { case succes of
				  true : chread="alphabet-character read";
				  false : chread undefined }
   end;


var
   x : integer;
   check : readstatus;
   syntax :syntaxtype;


procedure nextsymbol (var status : readstatus);
{ pre :: not( ([' '] in alfabet) and spacescount )
 post :: (succes and chread="alphabet-character read")
          and (not succes and
              ((eolnreached="EOLN in input") xor (eofreached="EOF in input")) )
          and (spacescount and
                   nspaces="#spaces read before alphabet-character/EOF/EOLN")
}

begin
  with status do begin
     succes := false;   { I :: succes=alphabet-character read }
     if eof then begin
        eofreached := true;
     end else begin
        if eoln then begin
           readln;
           eolnreached := true;
	end else begin
           if spacescount then begin
              nspaces := 0;
              if input^=' ' then begin
                 read(chread);   { chread=' ' }
                 nextsymbol(status);
                 nspaces := nspaces+1;
              end else begin
                 if input^ in alphabet then begin
                    read(chread);
                    succes := true;
                 end;
              end;
           end else begin
              if input^ in alphabet then begin
                  read(chread);
                  succes := true;
              end;
	   end;
	   if eoln then begin
	      readln;
	      eolnreached := true;
	   end else begin
	      eolnreached := false;
	   end;
	   eofreached := eof;
        end;
     end;
  end;
end; { nextsymbol }


procedure readinteger (var number : integer; var int_syntax : syntaxtype; var status : readstatus);

begin
  with status do begin
    alphabet := [ '0'..'9' ];
    spacescount := true;
    nextsymbol(status);
    if succes then begin
      number := ord(chread) - ord('0');
      if (nspaces=0) and (chread<>'0') then begin
        int_syntax := correct;
      end else begin
        int_syntax := wrong_format;
      end;
      spacescount := false;
      nextsymbol(status);
      while succes do begin
	 number := 10*number + ord(chread) - ord('0');
	 nextsymbol(status);
      end;
    end else begin
       int_syntax := incorrect;
    end;
  end;
end; { readinteger }


procedure reportstatus (status : readstatus);

begin
   with status do begin
      write('eofreached=', eofreached:5, ' ; ');
      write('eolnreached=', eolnreached:5, ' ; ');
      if spacescount then begin
	 write('nspaces=', nspaces:1, ' ; ');
      end;
      write('succes=', succes:5, ' ; ');
      if succes then begin
	 write('chread=', chread, ' ; ');
      end;
   end;
   writeln(' . ');
end; { reportstatus }



begin   { strong-testing }
   readinteger(x, syntax, check);
   case syntax of
     correct      : writeln('correct: ', x:1);
     wrong_format : writeln('wrong format: ', x:1);
     incorrect    : writeln('incorrect');
   end; { case }
   reportstatus(check);
   check.spacescount := true;
   check.alphabet := [ 'a'..'z', 'A'..'Z', '0'..'9' ];
   check.eofreached := false;
   while not check.eofreached do begin
      { read alphabet-characters }
      repeat
	 nextsymbol(check);
	 reportstatus(check);
      until not check.succes;
      { read EOLN/EOF's }
      repeat
	 nextsymbol(check);
	 reportstatus(check);
      until check.eofreached;
   end;
end.
